unit evm; {56002evm procedures}

interface
procedure LoadEvm(filename:string;baudrate:word;run:boolean);
procedure SetEvmPort(port:byte);
function EvmResult:integer;
function EvmMsg(errNr:integer):string;
{
0     Load OK
1     Can't find load file
2     Error reading fileheader
3     File is not a .CLD
4     File is not an absolute object file
5     More than 255 sections in the file
6     Error reading runtime header
7     EVM not responding
8     Error reading section header
9     Error reading section data
10    EVM did not acknowledge
}
implementation
uses crt,dos;

const MaxSectSize=255;
      MaxRecSize=8;
      Freloc=$0000001;   {relocation info stripped}
      Fexec =$0000002;   {executable file}
      Flnno =$0000004;   {line numbers stripped}
      Flsyms=$0000010;   {local symbols stripped}
      Fccof =$0010000;   {C-compiled object file}
      Pspace=0;
      Xspace=1;
      Yspace=2;
      Lspace=3;
      Sregular=$0000;    {section flags}
      Sdummy  =$0001;
      Snoload =$0002;
      Sgroup  =$0004;
      Spad    =$0008;
      Scopy   =$0010;
      Stext   =$0020;    {program code}
      Sdata   =$0040;    {initialised data}
      Sbss    =$0080;    {uninitialised data}
      Sblock  =$0400;    {block of repeated data}
      Sovlay  =$0800;
      Smacro  =$1000;
      signat  =$000002C5;  {signature at start of file}
      memtags:array[0..3] of byte=($15,$1B,$21,$27);{boot jumps for P,X,Y,L}
      runjump=$11;                        {boot jump for run}
      deadend=$3D;                        {boot jump for dead-end}
      memchars:array[0..4] of char=('P','X','Y','L','#');
      portadr:array[1..4] of word=($3F8,$2F8,$3E8,$2E8);
      msgs:array[0..10] of string=
      ('Load successful',
       'Can''t find load file',
       'Error reading fileheader',
       'File is not a .CLD',
       'File is not an absolute object file',
       'More than 255 sections in the file',
       'Error reading runtime header',
       'EVM not responding',
       'Error reading section header',
       'Error reading section data',
       'EVM did not acknowledge');

type Tmem=record
           Adr:longint;     {actual address}
           space:longint    {0,1,2,3,4=P,X,Y,L,none=memory-space}
          end;
TFilehdr=record
               Fmagic:longint;
               NrSect:longint;        {number of sections}
               DateTime:longint;      {date/timestamp unknown format}
               SymTabStart:longint;   {offset to start of symbol table}
               SymTabSize:longint;    {number of symble table entries}
               OptFilHdrSize:longint; {size of optional file header}
               Flags:longint;         { see page E-4 }
              end;
     TOptFH=record     {only the run-time optional header here}
             MagicNr:longint;        {not-used: zeroes?}
             Version:longint;        {not-used: zeroes?}
             TextSize:longint;       {size of executable code}
             InDataSize:longint;     {size of initialised data}
             UnDataSize:longint;     {size of unitinialised data}
             EntryPoint:Tmem;        {entry point}
             TextStart:Tmem;         {beginning address of executable code}
             DataStart:Tmem;         {beginning address of initialised data}
             TextEnd:Tmem;           {end address of executable code}
             DataEnd:Tmem            {end address of data}
            end;
     TSectHdr=record
               SectName:array[0..7] of char;
               PhysAdr:Tmem;     { run address }
               VirtAdr:Tmem;     { load address }
               SectSize:longint;
               DataStart:longint;    {offset in file to raw data}
               RelocStart:longint;   {offset in file to relocation table}
               LineNrStart:longint;  {offset in file to line number table}
               RelocSize:longint;    {size of relocation table}
               LineNrSize:longint;   {size of line number table}
               Flags:longint;        { see page E-9 }
              end;

var comport:byte;
    timer:word;
    evmerror:integer;

function evmresult:integer;
begin
 evmresult:=evmerror;
 evmerror:=0
end;

function EvmMsg(ErrNr:integer):string;
begin
 EvmMsg:=msgs[ErrNr]
end;

procedure timerset(var timer:word;ticks:word);
assembler;
asm
    xor ax,ax
    int $1A             {get tick count into dx}
    les bx,timer        {point to our timer}
    add dx,ticks        {add the required number of ticks to the tick count}
    mov es:[bx],dx      {save the result in our timer}
end;

function timeout(var timer:word):boolean;
assembler;
asm
    xor ax,ax
    int $1A          {get the tick count}
    xor al,al
    les bx,timer     {point to our timer}
    sub dx,es:[bx]   {subtract the tick count from our timer value}
    js @1
    inc al           {return true if timed-out}
@1:
end;

procedure longreverse(var block;size:longint);
type cast=array[0..32767] of byte;
var M,N:integer;
    temp:byte;
begin
 for N:=0 to (size div 4)-1 do for M:=0 to 1 do
 begin
  temp:=cast(block)[4*N+M];
  cast(block)[4*N+M]:=cast(block)[4*N+3-M];
  cast(block)[4*N+3-M]:=temp
 end
end;

procedure out1(data:byte);
begin
 repeat until (port[portadr[comport]+5] and $20)>0;
 port[portadr[comport]]:=data
end;

procedure out2(data:word);
begin
 out1(hi(data));
 out1(lo(data))
end;

procedure out3(data:longint);
begin
 out1(data shr 16);
 out1(data shr 8);
 out1(data);
end;

procedure initcom(baudrate:word);
var B:byte;
begin
 port[portadr[comport]+3]:=$83;    {8,n,1, access divider}
 portw[portadr[comport]]:=115200 div baudrate;   {set baudrate}
 port[portadr[comport]+3]:=$03;                   {8,n,1,access data}
 port[portadr[comport]+1]:=0;      {no interrupts}
 port[portadr[comport]+4]:=0;
 B:=port[portadr[comport]+5];
 B:=port[portadr[comport]]         {mop up any flags}
end;

procedure setevmport(port:byte);
begin
 comport:=port
end;

procedure loadevm(filename:string;baudrate:word;run:boolean);
var infile:file;
    Filehdr:Tfilehdr;
    OptFhdr:ToptFH;
    Sections:array[0..MaxSectSize-1] of ^Tsecthdr;
    SectionNr:byte;
    NrRead:word;
    RecSize:word;
    WordCount:Byte;
    WordsDone:word;
    RawData:array[0..MaxRecSize-1] of longint;

 procedure loaderror(errornumber:integer);
 begin
  evmerror:=errornumber;
  if filerec(infile).mode<>fmclosed then close(infile)
 end;

begin
 port[portadr[comport]+4]:=2;          {apply reset to EVM}
 timerset(timer,1);
 assign(infile,filename);
 {$I-}
 reset(infile,1);
 {$I+}
 if ioresult>0 then
 begin
  loaderror(1);
  exit
 end;
 blockread(infile,FileHdr,sizeof(FileHdr),NrRead);
 if NrRead<>sizeof(FileHdr) then
 begin
  loaderror(2);
  exit
 end;
 longreverse(FileHdr,sizeof(FileHdr));
 if FileHdr.Fmagic<>signat then
 begin
  loaderror(3);
  exit
 end;
 if FileHdr.OptFilHdrSize<>sizeof(ToptFH) then
 begin
  loaderror(4);
  exit
 end;
 if FileHdr.NrSect>MaxSectSize then
 begin
  loaderror(5);
  exit
 end;
 blockread(infile,OptFHdr,sizeof(OptFHdr),NrRead);
 if NrRead<>sizeof(OptFHdr) then
 begin
  loaderror(6);
  exit
 end;
 longreverse(OptFHdr,sizeof(OptFHdr));
 repeat until timeout(timer);           {make sure reset has been on for 55mS}
 initcom(1200);                         {remove reset and set up comport}
 timerset(timer,1);
 repeat until timeout(timer);           {wait for EVM to start}
 out2(round(625000/Baudrate)-1);        {send EVM baudrate}
 timerset(timer,5);
 repeat until timeout(timer) or ((port[portadr[comport]+5] and 1)>0); {wait for ack from DSP}
 if timeout(timer) then
 begin
  loaderror(7);
  exit
 end;
 wordcount:=port[portadr[comport]];
 initcom(Baudrate);      {set new baudrate}
 for SectionNr:=0 to FileHdr.NrSect-1 do
 begin
  new(Sections[SectionNr]);
  blockread(infile,Sections[SectionNr]^,sizeof(TSectHdr),NrRead);
  longreverse(Sections[SectionNr]^,sizeof(TSectHdr));
  if NrRead<>sizeof(TSectHdr) then
  begin
   loaderror(8);
   exit
  end;
 end;
 for SectionNr:=0 to FileHdr.NrSect-1 do
 begin
  with Sections[SectionNr]^ do
  begin
   if (SectSize>0) and (DataStart>0) then
   begin
    seek(infile,datastart);
    WordsDone:=0;
    while WordsDone<SectSize do
    begin
     RecSize:=SectSize-WordsDone;
     if RecSize>MaxRecSize then RecSize:=MaxRecSize;
     out1(byte(memtags[Physadr.space]));
     if Physadr.space=3 then out2(Physadr.adr+(WordsDone div 2))
      else out2(Physadr.adr+WordsDone);
      case 2*byte((Flags and Sblock)>0)+byte(Physadr.space=3) of
      0:   out2(RecSize);         {P,X,Y space}
      1:   out2(RecSize div 2);   {L space}
      2,3: out2(Virtadr.adr);     {blocks of any space}
     end;
     blockread(infile,Rawdata,RecSize*4,NrRead);
     if NrRead<>RecSize*4 then
     begin
      loaderror(9);
      exit
     end;
     longreverse(Rawdata,RecSize*4);
     repeat   {once for normal or (Virtadr.adr) times for blocks}
      if (Flags and Sblock)>0 then dec(Virtadr.adr);
      for  WordCount:=0 to RecSize-1 do
      begin
       out3(rawdata[WordCount]);
       inc(WordsDone)
      end
     until ((Flags and Sblock)=0) or (Virtadr.adr=0)
    end
   end
  end
 end;
 if run then
 begin
  out1(runjump);
  out2(OptFHdr.EntryPoint.adr);
  timerset(timer,10);
  repeat until timeout(timer) or ((port[portadr[comport]+5] and 1)>0);
  if timeout(timer) then
  begin
   loaderror(10);
   exit
  end
 end else out1(deadend);            {dead-end the DSP}
 wordcount:=port[portadr[comport]];
 loaderror(0)
end;

begin
 comport:=1;
 evmerror:=0;
end.