program psk31;           {psk31(evm) driver}
{

 Please contact the author peter.martinez@btinternet.com to report
 bugs or suggest changes. This is freeware for amateur use only

format of data from evm to PC:
PPPPPPP0: P is 7-bit fractional receive phase.            At 500Hz
00010011: X-off code from evm to stop PC                  When required
00010001: X-on code from evm to restart PC                When required

format of data from PC to evm:
000000TR: carrier on when T=1; phase-reverse when R=0.    by X-OnOff control
          these are bufferred in the EVM and processed at 31.25Hz
0001XXXX: not used
0010DDDD: shift D into lsb-end of 24-bit register         When required
0011XXXX: move 24-bit register to receive frequency       When required
0100XXXX: move 24-bit register to transmit frequency      When required
0101XXXX and higher not used

}

uses dos,crt,graph,evm;
const portadr:array[1..4] of word=($3F8,$2F8,$3E8,$2E8);
      picmask:array[1..4] of byte=($10,$08,$10,$08);
      intvec:array[1..4] of byte=(12,11,12,11);
      bufsize=4096;      {131 secs of received traffic}
      persist=10;        {tuning display persistence}
      samplerate=8000;   {evm sample rate for computing frequencies}
      dcdlimit=18;       {number of bits to test for carrier on/off detect}
      xoncode=$11;
      xoffcode=$13;      {codes expected from evm for flow control}
      onoff:array[0..1] of string[3]=('OFF','ON');

type keyproc=procedure(C:char);
     Tcode=(ascii,varicode);
     Tplot=record
      X,Y:word
     end;
     Tbuf=record
           data:array[0..bufsize-1] of byte;
           inptr,outptr:word
          end;

var psk31dir:dirstr;
    callsign:string[10];
    cwid:string[20];
    comportNr:byte;
    error:byte;    {used to look at evm loader errors}
    rxbuf:Tbuf;    {input buffer for stuff from evm}
    existcom:pointer; {com port interrupt vector to go back afterwards}
    delayline:array[0..15] of byte;   {32mS-worth of rx phase data}
    varsreg:word;                     {for decoding varicode}
    rxdata,bitcount,column,lastphase,truecount,falsecount:byte;
    dptr,bitclock:integer;
    init,boot,afc,neton,autolf,soundon,fileout:boolean;
    simplex,logon,txon,quit,toolong,dcd:boolean;
    rxfreq,txfreq:real;                    {frequencies in Hz}
    rxfstr:string[6];                      {rxfreq as shown on screen}
    outfile,logfile:text;
    filename:string[63];
    keydo:keyproc;
    code:Tcode;
    encotab:array[0..127] of word;      {encode table for varicode}
    decotab:array[0..511] of byte;      {decode table for varicode}
    echoline:string[42];                {echoes keyboard input}
    tune:array[0..persist-1] of Tplot;  {stores tuning display history}
    tuneptr:byte;                       {points to tuning display history}
    xinflag:byte;                       {stores flowcontrol codes from evm}

{$L egavga.obj}
procedure egavga; external;

procedure startgraph;
var GrDr,GrMo:integer;
begin
 GrDr:=registerbgidriver(@egavga);
 GrDr:= detect;
 initgraph(GrDr,GrMo,'');
 if graphresult<>0 then halt;
end;

function nextR(ptr:word):word;
begin
 nextR:=(ptr+1) mod bufsize
end;

procedure initvari(Fname:string);   {initialise varicode lookup tables}
var F:text;
    I:integer;
    W,mask:word;
    S:string[10];
begin
 assign(F,Fname);
 {$I+}
 reset(F);
 for I:=0 to 511 do decotab[I]:=240;    {prefill with error symbol}
 for I:=0 to 127 do
 begin
  readln(F,S);                          {get varicode string from file}
  W:=4;
  while S<>'' do                        {copy string chars into codeword bits}
  begin
   W:=W shl 1;
   if S[length(S)]='1' then inc(W);
   S:=copy(S,1,length(S)-1)
  end;
  encotab[I]:=W;                             {put codeword into encode table}
  mask:=$8000;
  while (mask and W)=0 do mask:=mask shr 1;  {find first non-zero bitposition}
  W:=(W and not(mask)) shr 1;                {zero it and shift out bit0}
  decotab[W]:=I                              {fill in decode table entry}
 end;
 close(F);
 {$I-}
end;

procedure setcode(C:Tcode);
const codestr:array[0..1] of string[12]=('F1: ASCII','F1: VARICODE ');
begin
 setcolor(black);
 if init then outtextXY(280,5,codestr[byte(code)]);
 code:=C;
 setcolor(white);
 outtextXY(280,5,codestr[byte(code)]);
end;

procedure setafc(A:boolean);
begin
 setcolor(black);
 if init then outtextXY(345,20,onoff[byte(afc)]);
 afc:=A;
 if afc then setcolor(lightgreen) else setcolor(lightred);
 outtextXY(345,20,onoff[byte(afc)]);
end;

procedure setnet(N:boolean);
begin
 setcolor(black);
 if init then outtextXY(565,5,onoff[byte(neton)]);
 neton:=N;
 if neton then
  begin
   setcolor(lightgreen);
   rxfstr:='' {force frequency update}
  end else setcolor(lightred);
 outtextXY(565,5,onoff[byte(neton)]);
end;

procedure setdcd(D:boolean);
begin
 setcolor(black);
 if init then outtextXY(455,5,onoff[byte(dcd)]);
 dcd:=D;
 if dcd then setcolor(lightgreen) else setcolor(lightred);
 outtextXY(455,5,onoff[byte(dcd)]);
end;

procedure setfile(S:boolean);
begin
 fileout:=S;
 if fileout then setcolor(lightgreen) else setcolor(black);
 outtextXY(350,50,'FILE SEND');
end;

procedure setlog(L:boolean);
begin
 setcolor(black);
 if init then outtextXY(455,35,onoff[byte(logon)]) else logon:=false;
 if L then
 begin
  assign(logfile,psk31dir+'PSK31.LOG');
  {$I-}
  append(logfile);
  {$I+}
  if ioresult>0 then rewrite(logfile)
 end else
 begin
  if logon then close(logfile)
 end;
 logon:=L;
 if logon then setcolor(lightgreen) else setcolor(lightred);
 outtextXY(455,35,onoff[byte(logon)])
end;

procedure setsound(S:boolean);
begin
 soundon:=S;
 if soundon then sound(round(rxfreq+0.5)) else nosound
end;

procedure scroll;
var R:registers;
begin
 R.ax:=$0601;     {scroll one "line" of 16 pixels}
 R.bx:=$0000;     {black}
 R.ch:=0;         {from top}
 R.cl:=0;         {from left}
 R.dh:=24;        {to bottom}
 R.dl:=79;        {to right}
 intr($10,R);     {get BIOS to do it}
end;

procedure linefeed;
begin
 if whereY=25 then scroll else write(#10)
end;

procedure writescreen(C:char);
begin
 if logon then
 begin
 {$I-}
  if (C<>#10) or not(autolf) then write(logfile,C);
  if (C=#13) and autolf then write(logfile,#10);
 {$I+}
  if ioresult>0 then setlog(false)
 end;
 case C of
   #8: write(#8' '#8);
  #10: if not(autolf) then linefeed;
  #13: begin
       write(#13);
       if autolf then linefeed;
      end;
   #32..#127,#240:
     begin
      if whereX=80 then
      begin
       write(#13);
       if autolf then linefeed;
      end;
      write(C)
     end
 end
end;

procedure writerx(B:byte);
begin
 if dcd and not(simplex and txon) then writescreen(char(B))
end;

procedure dodcd(bit:boolean);
begin
 if bit then
 begin
  truecount:=0;
  if falsecount<dcdlimit then
  begin
   inc(falsecount);
   if falsecount=dcdlimit then setdcd(false)
  end
 end
  else
 begin
  falsecount:=0;
  if truecount<dcdlimit then
  begin
   inc(truecount);
   if truecount=dcdlimit then setdcd(true)
  end
 end
end;

procedure decode9(bit:boolean);
begin
 case bitcount of
  8:    begin
         if bit then                  {stop bit}
         begin
          writerx(rxdata shr 1);
          bitcount:=0
         end
        end;
  1..7: begin                          {7 data bits}
         rxdata:=(rxdata shr 1);
         if bit then rxdata:=rxdata+(byte(bit) shl 7);
         inc(bitcount)
        end;
  0: begin
      rxdata:=0;
      if not(bit) then bitcount:=1     {startbit}
     end;
 end;
end;

procedure decodeV(bit:boolean);
begin
 varsreg:=varsreg shr 1;                     {shift previous bits}
 if varsreg and $0008>0 then toolong:=true;  {mark codes length>10}
 if bit then varsreg:=varsreg or $8000;      {add in this bit}
 if ((varsreg and $C000)=0) and (varsreg<>0) then      {if end of code}
 begin
  while (varsreg and $0001)=0 do varsreg:=varsreg shr 1;  {normalise to lsb}
  varsreg:=varsreg shr 1;                                 {shift out first 1}
  if toolong then writerx(240) else writerx(decotab[varsreg]);  {lookup and out}
  toolong:=false;                                            {reset flag}
  varsreg:=0                                                 {reset shiftreg}
 end
end;

procedure decode(bit:boolean);    {reversal=false}
begin
 dodcd(bit);
 case code of
  ascii:    decode9(not(bit));        {reversal=logic 1}
  varicode: decodev(bit);             {reversal=logic 0}
 end
end;

procedure tunescope(dph:byte);
begin
 setcolor(black);
 circle(tune[tuneptr].X,tune[tuneptr].Y,3);     {blank out last spot}
 circle(tune[tuneptr].X,tune[tuneptr].Y,2);
 tune[tuneptr].X:=72+round(32*cos(pi*dph/128));
 tune[tuneptr].Y:=39-round(32*sin(pi*dph/128));  {calc coords of new spot}
 if dcd then setcolor(yellow) else setcolor(red);
 circle(tune[tuneptr].X,tune[tuneptr].Y,3);     {show it}
 circle(tune[tuneptr].X,tune[tuneptr].Y,2);
 tuneptr:=(tuneptr+1) mod persist               {count round persistence buffer}
end;

procedure doafc(dph:byte);
var pp:integer;
begin
 pp:=((dph-64) and $7F)-64;      {pp in range -64 to +63}
 if pp=-64 then pp:=0;           {eliminate slight bias}
 rxfreq:=rxfreq-pp*0.0002;       {tweek rxfreq a tiny bit}
end;

procedure receive;
var nowphase,diffphase:byte;
    oldclock:longint;
begin
 if rxbuf.inptr=rxbuf.outptr then exit;      {nothing to do}
 nowphase:=rxbuf.data[rxbuf.outptr];         {get channel phase from buffer}
 rxbuf.outptr:=nextR(rxbuf.outptr);
 diffphase:=delayline[dptr]-nowphase;        {form phase difference}
 delayline[dptr]:=nowphase;                  {update delayline}
 dptr:=(dptr+1) mod 16;                      {count round delayline}
 oldclock:=bitclock;
 bitclock:=bitclock+2048;                    {add half bitrate}
 if abs(nowphase-lastphase)>$40 then         {if phase has swapped in last 2mS}
 begin
  lastphase:=nowphase;
  bitclock:=round(bitclock*0.98);            {nudge the bitclock into phase}
 end;
 bitclock:=bitclock+2048;                    {add 2nd-half of bitrate}
 diffphase:=diffphase+$40;                   {..so databit is in msb}
 if abs(bitclock-oldclock)>32768 then        {if bitcentre}
 begin
  decode(diffphase<128);
  tunescope(diffphase);
  if dcd and afc and not(txon) then doafc(diffphase)
 end
end;

procedure sendevm(data:byte);
begin
 while ((xinflag=xoffcode) and ((data and $70)=0)) { traffic and Xoff}
  do receive;                                      {..do hang about}
 port[portadr[comportNr]]:=data;                              {send data}
 while (port[portadr[comportNr]+5] and $20)=0 do receive
end;

procedure settxfreq;        {show tx frequency same as rx frequency}
var S:string[20];
begin
 str(txfreq:6:1,S);
 S:='   '+S+'   ';
 setcolor(black);          {blank out the old one}
 if init then outtextXY(150,20,S);
 txfreq:=rxfreq;           {set the new one}
 str(txfreq:6:1,S);
 S:='TX='+S+' Hz';
 setcolor(white);
 outtextXY(150,20,S)       {show it}
end;

procedure updatefreq;
type cast=record
           low,mid,high,top:byte    {to get at the top byte in a longint}
          end;
var newfreq:string[6];
    S:string[12];
    rxflong:longint;
    I:integer;
begin
 str(rxfreq:6:1,newfreq);
 if newfreq=rxfstr then exit;    {close enough.. don't bother}
 S:='   '+rxfstr+'   ';
 setcolor(black);
 if init then OuttextXY(150,5,S);             {blank out old frequency}
 S:='RX='+newfreq+' Hz';
 setcolor(white);
 OuttextXY(150,5,S);                          {write in new frequency }
 rxflong:=round(rxfreq/samplerate*256*65536); {form new freq as binary}
 for I:=0 to 5 do
 begin
  rxflong:=rxflong shl 4;                      {shift nybble into top byte}
  sendevm($20+((cast(rxflong).top) and $F)); {send it with command $2*}
 end;
 sendevm($30);                                 {send 'update rxf' command}
 if neton then
 begin
  sendevm($40);                                {send 'update txf' command}
  settxfreq                                    {paint in new tx freq}
 end;
 rxfstr:=newfreq;                               {keep for next update}
 if soundon then setsound(soundon)
end;

procedure setptt(T:boolean);
var I,J:integer;
begin
 if fileout then
 begin
  write(#7);    {don't mess with PTT while sending a file}
  exit
 end;
 setcolor(black);
 if init then outtextXY(450,20,onoff[byte(txon)]) else txon:=false;
 if T then setcolor(lightgreen) else setcolor(lightred);
 outtextXY(450,20,onoff[byte(T)]);
 if T=txon then exit;
 if T then J:=2 else J:=3;       {revs at turn-on: carrier at turn-off}
 for I:=0 to 2*dcdlimit do sendevm(J);
 if not(T) then sendevm(0);                {finish with txoff to kill tone}
 txon:=T;
end;

procedure sendcw(C:char);
const cwtxtab:array[32..93] of byte=       {ASCII to morsecode }
      ($80,$AC,$4A,$16,$00,$00,$00,$7A,$B4,$B6,$54,$14,$CE,$86,$56,$94,
       $FC,$7C,$3C,$1C,$0C,$04,$84,$C4,$E4,$F4,$00,$AA,$00,$8C,$44,$32,
       $00,$60,$88,$A8,$90,$40,$28,$D0,$08,$20,$78,$B0,$48,$E0,$A0,$F0,
       $68,$D8,$50,$10,$C0,$30,$18,$70,$98,$B8,$C8,$58,$E8,$6C);
var I:integer;
    symbol:byte;
begin
 if not(C in [#32..#93]) then exit;
 symbol:=cwtxtab[byte(C)];
 while symbol>0 do
 begin
  case symbol of
        $80: for I:=0 to 1 do sendevm(0);   {letter gap}
   $81..$FF: for I:=0 to 5 do sendevm(3);   {dash}
   $01..$7F: for I:=0 to 1 do sendevm(3);   {dot}
  end;
  for I:=0 to 1 do sendevm(0);   {element gap}
  symbol:=symbol shl 1;
 end
end;

procedure cwident;
var I:integer;
begin
 setPTT(false);
 for I:=1 to length(cwid) do sendcw(upcase(cwid[I]))
end;

procedure sendascii(C:char);
var I:integer;
    B:byte;
begin
B:=byte(C) or $80;      {add stop bit}
 sendevm(3);            {start bit}
 repeat
  if odd(B) then sendevm(2) else sendevm(3);
  B:=B shr 1
 until B=0        {send bits until stopbit has gone}
end;

procedure sendvari(C:char);
var I:integer;
    W:word;
begin
 W:=encotab[byte(C)];
 repeat
  if odd(W) then sendevm(3) else sendevm(2);
  W:=W shr 1
 until W<2
end;

procedure sendchar(C:char);
begin
 case code of
  ascii:    sendascii(C);
  varicode: sendvari(C)
 end
end;

procedure echo(C:char);
begin
 if simplex then writescreen(C);
 if C=#13 then C:=#27;
 if C=#10 then C:=#25;   {convert CR and LF to symbols}
 setcolor(black);
 outtextXY(145,65,echoline);                          {blank out old line}
 if C=#8 then echoline:=' '+copy(echoline,1,41) else
  echoline:=copy(echoline,2,41)+C;                    {add this character}
 setcolor(yellow);
 outtextXY(145,65,echoline)                           {show new line}
end;

procedure sendtraffic(C:char); far;     {called via keydo procedure variable}
var L:longint;
    data:byte;
begin
 if not(txon) then exit;          {don't transmit if tx is off}
 echo(C);
 if (C=#10) and autolf then exit;
 if (C=' ') and (column>70) then
 begin
  sendchar(#13);
  if not(autolf) then sendchar(#10);
  column:=0;
  exit;
 end;
 if C in [#32..#126] then inc(column);
 if C=#13 then column:=0;
 sendchar(C);
end;

procedure dofilename(C:char); far;  {called via keydo procedure variable}
begin
 setcolor(black);
 outtextXY(400,50,filename);         {blank out filename}
 case C of
  #13: begin                         {enter}
        keydo:=sendtraffic;          {switch back to traffic}
        outtextXY(350,50,'FILE:');
        if filename='' then exit;
        assign(outfile,filename);    {try and open the file}
        {$I-}
        reset(outfile);
        {$I+}
        if ioresult=0 then setfile(true);    {turn on file-send if opened OK}
        exit
       end;
  #33..#126: filename:=filename+upcase(C);   {filename character}
  #8: begin                                                {backspace}
       filename:=copy(filename,1,length(filename)-1)
      end;
  #27: begin                                         {escape}
        keydo:=sendtraffic;                  {switch back to traffic}
        outtextXY(350,50,'FILE:');           {blank out legend}
        exit
       end;
 end;
 setcolor(white);
 outtextXY(400,50,filename)         {show filename so far}
end;

procedure sendfilechar;
var C:char;
begin
 if not(fileout) then exit;       {nothing to do}
 if eof(outfile) then
 begin
  close(outfile);                 {shut it if finished it}
  setfile(false);
  exit
 end;
 read(outfile,C);
 column:=0;           {prevent auto-newline screwing things up}
 sendtraffic(C)
end;

procedure CQcall;
const kk:string[8]='pse KK'#13#10;
var I,J:integer;
    CQstr:string;
begin
 CQstr:='CQ CQ CQ de '+callsign+' '+callsign+' '+callsign+#13#10;
 setPTT(true);                                       {turn on the tx}
 sendtraffic(#13);
 sendtraffic(#10);
 for J:=0 to 2 do
 for I:=1 to length(CQstr) do sendtraffic(CQstr[I]); {send the call}
 for I:=1 to length(kk) do sendtraffic(kk[I]);
 CWident
end;

procedure dokey;
var C:char;
    I:integer;
begin
 if not(keypressed) then
 begin
  sendfilechar;
  exit
 end;
 C:=readkey;
 case upcase(C) of
  #0: begin                 {process function/cursor keys}
       case readkey of
{F1}    #59: if code=ascii then setcode(varicode) else setcode(ascii);
{F2}    #60: setdcd(not(dcd));
{F3}    #61: setnet(not(neton));
{F4}    #62: setafc(not(afc));
{F5}    #63: setptt(not(txon));
{F6}    #64: cwident;
{F7}    #65: setsound(not(soundon));
{F8}    #66: setlog(not(logon));
{F9}    #67: begin
              setfile(false);
              keydo:=dofilename;
              setcolor(lightcyan);
              outtextXY(350,50,'FILE:');
              filename:=''
             end;
{F10}   #68: quit:=true;
        #75: rxfreq:=rxfreq-1;        {left arrow}
        #115: rxfreq:=rxfreq-10;      {control left-arrow}
        #77: rxfreq:=rxfreq+1;        {right arrow}
        #116: rxfreq:=rxfreq+10;      {control right-arrow}
{altF1} #104: CQcall
       end
      end;
  else if not(fileout) then keydo(C)
 end
end;

procedure evminput; interrupt;
var B:byte;
begin
 if (port[portadr[comportNr]+5] and 1)>0 then
 begin
  B:=port[portadr[comportNr]];
  if (B and 1)=0 then          {if it is a phase value..}
  begin
   rxbuf.data[rxbuf.inptr]:=B;        {..put it in the buffer}
   rxbuf.inptr:=nextR(rxbuf.inptr);
  end
  else if (B=xoffcode) or (B=xoncode) then xinflag:=B  {set flowcontrol flag}
 end;
 port[$20]:=$20                  {clear the PIC}
end;

procedure initcom(baudrate:word);
var B:byte;
begin
 xinflag:=xoncode;
 port[portadr[comportNr]+3]:=$83;    {8,n,1, access divider}
 portw[portadr[comportNr]]:=115200 div baudrate;   {set baudrate}
 port[portadr[comportNr]+3]:=$03;                   {8,n,1,access data}
 port[portadr[comportNr]+1]:=1;      {rx interrupt only}
 port[portadr[comportNr]+4]:= port[portadr[comportNr]+4] or 8;    {enable irq}
 B:=port[portadr[comportNr]+5];
 B:=port[portadr[comportNr]];         {mop up any old data or errorflags}
 getintvec(intvec[comportNr],existcom);      {save pre-existing irq vector}
 setintvec(intvec[comportNr],@evminput);     {put ours in it's place}
 port[$21]:=port[$21] and not(picmask[comportNr]);    {enable PIC input}
 rxbuf.inptr:=0;
 rxbuf.outptr:=0                                    {set rxbuf empty}
end;

procedure closecom;
begin
 port[portadr[comportNr]+4]:=0;    {disable irq}
 port[portadr[comportNr]+1]:=0;      {rx interrupt off}
 port[$21]:=port[$21] or picmask[comportNr];     {turn off PIC input}
 setintvec(intvec[comportNr],existcom)     {put back the pre-existing irq vector}
end;

function caps(S:string):string;
var I:integer;
begin
 for I:=1 to length(S) do S[I]:=upcase(S[I]);
 caps:=S
end;

procedure doparam(P:byte);
var S:string;
    error:word;
begin
 S:=paramstr(P);
 if S[1] in ['/','-'] then
 case upcase(S[2]) of
  'C': callsign:=copy(S,3,255);
  'P': comportNr:=byte(S[3]) and 15;
  'B': boot:=true;
  'F': begin
        S:=copy(S,3,255);
        val(S,rxfreq,error);
        if error>0 then rxfreq:=1000
       end;
  'S': simplex:=true;
  'N': autoLF:=false
 end
end;

procedure doparams;
var parmNr:byte;
begin
 for parmNr:=1 to paramcount do doparam(parmNr)
end;

procedure setpsk31dir;
var N:namestr;
    E:extstr;
begin
 fsplit(paramstr(0),psk31dir,N,E)
end;

begin
 if pos('?',paramstr(1))>0 then
 begin
  writeln('31 baud PSK driver program for DSP56002EVM by G3PLX');
  writeln('Available commandline parameters:');
  writeln('/c followed by your callsign  e.g. /cG3PLX');
  writeln('/p followed by your COM port  e.g. /p2 for COM2');
  writeln('/b to load PSK31.CLD with G3PLX BOOT ROM');
  writeln('/f followed by centre frequency   e.g.  /f800.0');
  writeln('/s for simplex (on tx: mute rx and show local copy)');
  writeln('/n to disable auto-linefeed');
  writeln('Defaults: NoCall, COM1, no boot, 1000 Hz, duplex, autoLF');
  writeln;
  halt
 end;
 comportNr:=1;
 callsign:='NoCall';
 boot:=false;
 init:=false;
 setpsk31dir;
 rxfreq:=1000;
 rxfstr:='';
 simplex:=false;
 autolf:=true;
 doparams;
 txfreq:=rxfreq;
 cwid:='de '+callsign;
 setevmport(comportNr);
 if boot then
 begin
  loadevm(psk31dir+'psk31.cld',19200,true);  {load at 19200 baud and start it}
  error:=evmresult;
  if error>0 then
  begin
   writeln(evmmsg(error));         {print out what was wrong}
   halt(1)
  end
 end;
 initvari(psk31dir+'alphabet.dat');
 if ioresult>0 then
 begin
  writeln('Unable to load varicode alphabet file');
  halt
 end;
 initcom(19200);
 quit:=false;
 soundon:=false;
 varsreg:=0;
 toolong:=false;
 dptr:=0;
 bitclock:=0;
 bitcount:=0;
 truecount:=0;
 falsecount:=0;
 startgraph;
 setviewport(0,400,639,479,clipoff);
 keydo:=sendtraffic;
 echoline:='                                          ';
 echo(' ');
 directvideo:=false;       {enable BIOS text on graphics screen}
 highvideo;                {nice and bright}
 setcolor(lightblue);
 circle(72,39,28);
 circle(72,39,27);
 circle(72,39,36);
 circle(72,39,37);
 for tuneptr:=0 to persist-1 do
 begin
  tune[tuneptr].X:=60;            {initialise history}
  tune[tuneptr].Y:=40
 end;
 tuneptr:=0;
 setcolor(white);
 line(72,11,72,67);
 outtextXY(500,5,'F3: NET');
 outtextXY(500,20,'F6: cwid');
 outtextXY(280,20,'F4: AFC');
 outtextXY(280,35,'F7: sound');
 outtextXY(390,5,'F2: DCD');
 outtextXY(390,20,'F5: TX');
 outtextXY(390,35,'F8: LOG');
 outtextXY(500,35,'F9: file');
 outtextXY(500,65,'F10: quit');
 setcolor(darkgray);
 rectangle(0,0,639,79);
 rectangle(140,60,483,75);
 setcode(varicode);
 setfile(false);
 setlog(false);
 setafc(false);
 setnet(true);
 setdcd(false);
 setptt(false);
 column:=0;
 init:=true;
 repeat
  receive;
  dokey;
  updatefreq
 until quit;
 if boot then loadevm(' ',19200,false);  {reset the evm}
 nosound;
 restorecrtmode;
 closecom
end.